// Includes: guidelines
// 
// -Includes are necessary in cases a file uses identifiers that are declared/defined in other files. 
// -Includes are transitive, i.e. a file that is inluded in an included file is included. 
//    That creates two problems: 
//    1. As an identifier can only be defined once in the same scope, care should be taken that things are not redeclared/redefined 
//       when processing the whole chain of inclusion for each compilation unit. 
//    2. While you figure out the inlcusion list for a file, you might be tempted to *rely* on an included file's inclusions. Don't, 
//       because it creates confusion, and problems later if the included files change their list of inclusions. 
// -For the sake of clarity/simplicity, as a general rule, include only the *minimum* of what is *absolutely required* for the file to compile: 	
//    1. If you include a library that uses namespaces, use the "uses namespace_id::unit_id;" (e.g. "using std::cout;") notation preferably. 
//       "using namespace namespace_id" (e.g. "using namespace std;") works as well, but is less precise and may lead to confusion. 
//    2. When dealing with user-defined types/classes declared/defined in header files, you have the two following choices. 
//       The second choice is more specific and should be used preferably when possible.  
//       A. You may rely on inclusion of the header file (e.g. "#include "Player.h") where the class has been declared. The possible problem 
//          with that is that including a class declaration brings its own whole list of includes, and possibly some problems/confusion. 
//          One of these problems is the possibility of circular class dependencies. To avoid these, rely on choice B. whenever possible     
//       B. You may rely on a forward declaration - "class class_id;" (e.g. "class Player;"). Forward class declarations can only be used 
//          when the required type/class is referred to as unqualified (i.e. not using *, nor &), and none of the members of the class are 
//          referred to. This should be used preferably to (A), which is going to reduce the number of possible inclusion problems 
//          such as circular dependencies.   

#include <iostream>  // main uses cout
using std::cout;        
#include <vector>    // main uses the vector class and many of its member functions
using std::vector; 
#include "Player.h"  // main uses the Player class and calls its constructor, so the whole class is needed 
#include "Country.h" // main uses the Country class and calls its constructor, so the whole class is needed 

// -These are global variables used only by the main function.
// -They are not used by the Player/Country classes. 
// -So they only need to be declared/defined in this compilation unit. 
// -They could be declared/defined along with the respective classes, but then they 
//    generally would be in the cpp file (or else they would get redeclared 
//    if the header file is included more than once in the whole application). 
//    If they are in the classes' cpp files, then any other compilation unit 
//    that wants to use them would need to declare the as "extern", so that 
//    they are not redeclared/redefined, and they can be linked across compilation units. 

const int numPlayers = 4;
const int numCountries = 13;

// Uses the vector template and some of its member functions (constructor, at and push_back), as well as cout, 
//   which is in the iostream library. It does not use the string class. 
int main() {
	// Use of Player* would require only a forward declaration
	vector<Player*> allPlayers;
	vector<Country*> allCountries;
	for (int i = 1; i <= numPlayers; i++)
		allPlayers.push_back(new Player());
	for (int j = 0; j <= numCountries - 1; j++) {
		allPlayers.at(j % numPlayers)->ownedCountries.push_back(new Country(allPlayers.at(j % numPlayers)));
	}
	for (int k = 0; k <= numPlayers - 1; k++)
		cout << *allPlayers.at(k);
}

//////////// Question 23 : Risk /////////////////////////////
// 1. RiskDriver.cpp//////////////////////////////////// 4  /
//    main function unchanged                         // 2  /
//    global constants                                // 1  /
//    includes: vector, iostream, Player.h, Country.h // 1  /
// 2. Country.h //////////////////////////////////////// 4  /
//    include guard                                   // 1  /
//    includes: string, Player.h, iostream            // 1  /
//    no inline                                       // 1  /
//    operator<< declaration (free)                   // 1  /
// 3. Country.cpp ////////////////////////////////////// 4  /
//    includes: Country.h, Player.h,iostream, string  // 1  /
//    function definitions (Country::)                // 1  /
//    free operator<<                                 // 1  /
//    static member initialization                    // 1  /
// 4. Player.h ///////////////////////////////////////// 4  /
//    include guard                                   // 1  /
//    include: vector, string, iostream               // 1  /
//    forward declaration: Country                    // 2  /
//    no inline                                       // 1  /
// 5. Player.cpp /////////////////////////////////////// 4  /
//    includes: Player.h, Country.h, iostream string  // 1  /
//    function definitions (Player::)                 // 1  /
//    free operator<<                                 // 1  /
//    static member initialization                    // 1  /
//////////////////////////////////////////////////////// 20 /